<?php

/**
 * String utilities
 *
 * This file is part of the Brim project.
 * The brim-project is located at the following 
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 * 
 * <pre> Enjoy :-) </pre>
 *
 * @author Barry Nauta - July 2003
 * @package org.brim-project.framework
 * @subpackage util
 *
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2005 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php 
 * The GNU Public License
 */
class StringUtils
{
	/**
	 * Empty constructor
	 */
	function StringUtils ()
	{
	}

	/**
	 * Replaces quotes by the tag &amp;quote;
	 *
	 * @param string string the input string
	 * @return string the url encoded string
	 */
	function urlEncodeQuotes ($string)
	{
		$s1 = ereg_replace ("\"", "&quot;", $string);
		$s2 = ereg_replace ("\'", "&#39;", $s1);
		return $s2;
	}

	/**
	 * Replaces ampersands by the tag &amp;amp;
	 *
	 * @param string string the input string
	 * @return string the url encoded string
	 */
	function urlEncodeAmpersands ($string)
	{
		$s1 = ereg_replace ("&", "&amp;", $string);
		return $s1;
	}

	/**
	 * Checks whether the input starts with a given string
	 *
	 * @param string inputString the total String
	 * @param string startsWith a partial String
	 * @return true if 'inputString' starts with 'startsWith', false
	 * otherwise or if 'startsWith' is <code>null</code>
	 */
	function startsWith ($inputString, $startsWith)
	{
		if ($startsWith == null)
		{
				return false;
		}
	  	return (substr ($inputString, 0, strlen($startsWith)) 
			== $startsWith);
	}

	/**
	 * Removes newlines characters from the input string
	 *
	 * @param string string the input string
	 * @return string the input string without the newlines
	 * @deprecated
	 */
	function stripNewlines ($string)
	{
		$s1 = ereg_replace ("\n", "", $string);
		$s2 = ereg_replace ("\r", "", $s1);
		return $s2;
	}


	/**
	 * Replaces newline characters by br tages
	 *
	 * @param string string the input string
	 * @return string the input string without the replaced newlines
	 */
	function newlinesToHtml ($string)
	{
		$s1 = ereg_replace ("\n", "<br />", $string);
		$s2 = ereg_replace ("\r", "", $s1);
		return $s2;
	}

	/**
	 * Returns the property of the inputString. This assumes that the
	 * inputString is in the format of property=value.
	 *
	 * The function will take the length of the property + 1 
	 * (for the '=' sign) and returns the inputString without the 
	 * first characters, found by the length
	 *
	 * Calling the function getProperty ("property=value", "property") 
	 * will return "value"
	 *
	 * @return string the property as specified, <code>null</code>
	 * 		if either inputString or property is null
	 */
	function getProperty ($inputString, $property)
	{
		if ($inputString == null || $property == null)
		{
			return null;
		}
	  	return (substr ($inputString, 
	  		strlen($property)+1, 
			strlen($inputString)));
	}

	/**
	 * Truncates the inputstring at the given length and appands the 
	 * optional string.
	 *
	 * @param string inputString the inputstring
	 * @param integer length
	 * @param string append optional append characters, default to '...'
	 */
	function truncate ($inputString, $length, $append="...")
	{
		$leninput =  strlen ($inputString);
		$lenappend = strlen ($append);
		if (($leninput > $length) && ($leninput > $lenappend))
		{
			return substr ($inputString, 0, $length-$lenappend).$append;
		}
		else
		{
			return $inputString;
		}
	}

	/**
	 * Call the addslashes on the input string, only if get_magic_quotes
	 * is not set
	 *
	 * @param string input the string we would like to evaluate
	 * @return string the optionally modified input string
	 */
	function gpcAddSlashes ($input)
	{
		 return(get_magic_quotes_gpc() == 1 ?
		 	$input : addslashes($input));
	}


	/**
	 * Call the stripslashes on the input string, only if 
	 * get_magic_quotes is not set
	 *
	 * @param string input the string we would like to evaluate
	 * @return string the optionally modified input string
	 */
	function gpcStripSlashes ($input)
	{
		 return(get_magic_quotes_gpc() == 1 ?
		 	stripslashes($input) : $input);
	}

	/**
	 * Call the stripslashes on the input string or input array, 
	 * only if get_magic_quotes is not set. Calling this function 
	 * when the input is an array recursively calls the gpcStripSlashes
	 * on each member of the array. Calling this function on a 
	 * string results in a call to the gpcStripSlashes
	 *
	 * @param object input either string or array we would like to evaluate
	 * @return string or array the optionally modified input 
	 */
	function gpcStripSlashesDeep ($input)
       	{
	        $input = is_array($input) ?
	            array_map('gpcStripSlashesDeep', $input) :
	                gpcStripSlashes ($input);
	        return $input;
	}

	/**
	 * Returns a (almost) random string with with given length
	 *
	 * @param integer len thelength of the random string we would 
	 * like to have
	 * @return string the random string with requested length
	 */
	 function randomString($len)
	 {
	 	$i = 0;
		srand(date("s"));
		while($i<$len)
		{
			$str.=chr((rand()%26)+97);
			$i++;
		}
		$str=$str.substr(uniqid (""),0,22);
		return $str;
	}

	function removeBizarreCharacters ($input)
	{
		if (isset ($input) && $input != '')
		{
			// replace special characters
			$input = str_replace('ä', '�', $input);
			$input = str_replace('Ä', '�', $input);
			$input = str_replace('ö', '�', $input);
			$input = str_replace('ü', '�', $input);
			$input = str_replace('Ü', '�', $input);
			$input = str_replace('é', '�', $input);
			$input = str_replace('É', '�', $input);
			$input = str_replace('è', '�', $input);
			$input = str_replace('È', '�', $input);
			$input = str_replace('à', '�', $input);
			$input = str_replace('À', '�', $input);
			$input = str_replace('§', '�', $input);
			$input = str_replace('°', '�', $input);
			$input = str_replace('¦', '�', $input);
			$input = str_replace('ç', '�', $input);
			$input = str_replace('ô', '�', $input);
			$input = str_replace('Ô', '�', $input);
			$input = str_replace('û', '�', $input);
			$input = str_replace('Û', '�', $input);
			$input = str_replace('î', '�', $input);
			$input = str_replace('Î', '�', $input);
			$input = str_replace('�', '�', $input);
			$input = str_replace('è', '�', $input);
			$input = str_replace('\\', '\\\\', $input);
		}
		return $input;
	}
}
?>
