<?php
require_once('plugins/banking/model/Banking.php');
require_once('plugins/banking/model/BankingFactory.php');
require_once('framework/model/Services.php');
require_once('plugins/banking/model/CancellistServices.php');

/**
 * Services for Banking
 *
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Thomas Sterren - December 2004
 * @package org.brim-project.plugins.banking
 * @subpackage model
 * @filesource
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2005 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */
class BankingServices extends Services
{
	var $cancellistServices;

	/**
	 * Default constructor
	 */
	function BankingServices()
	{
		parent::Services();
		$this->itemFactory = new BankingFactory();

		$this->cancellistServices = new CancellistServices();

		include ('plugins/banking/sql/bankingQueries.php');
		$this->queries = $queries;
	}

	/**
	 * Adds information for an e-banking account
	 *
	 * @param integer userId the identifier for the user
	 * @param object item the e-banking information to be added
	 *
     * @return integer last know id. This is the id of the e-banking information
	 * that was newly inserted and for which an id has automatically
	 * been assigned.
	 */
	function addItem($userId, $item)
	{
		$now = date("Y-m-d H:i:s");

		if (!isset($item->guid))
		{
			$item->guid = $this->itemFactory->createGUID();
		}

		// execute the query
		$query = sprintf($this->queries['addItem'],
			$userId,
			$item->parentId,
			$item->isParent,
			addslashes(trim($item->name)),
			addslashes(trim($item->description)),
			addslashes(trim($item->visibility)),
			addslashes(trim($item->category)),
			addslashes($item->isDeleted),
			$now,
			$item->guid,
			addslashes(trim($item->login)),
			addslashes(trim($item->url)),
			addslashes(trim($item->contact_person)),
			addslashes(trim($item->contact_phone)));

		$this->db->Execute($query) or
				die ("Could not add bank.".
				$this->db->ErrorMsg () . " " . $query . '<br>');

		// fetch the last known id
		// todo this is generic code and is candidate to move to the base Services class
		$query = $this->queries['lastItemInsertId'];
		$result = $this->db->Execute($query) or
			die ("Could not retrieve last insert id for bank. ".
			$this->db->ErrorMsg () . " " . $query . '<br>');

        return $result->fields[0];
	}

	/**
	 * Modifies information for the e-banking account
	 *
 	 * @param integer userId the identifier for the user who modifies a e-banking account
	 * @param object item the modified e-banking information
	 */
	function modifyItem($userId, $item)
	{
		//die ("BankingService::modifyItem ".bprint_r($item));
		$now = date ("Y-m-d H:i:s");

		$query = sprintf ($this->queries['modifyItem'],
			$now,							// when_modified
			addslashes($item->name),
			$item->isDeleted,
			addslashes($item->login),
			addslashes($item->url),
			addslashes($item->contact_person),
			addslashes($item->contact_phone),
			addslashes($item->description),
			$item->itemId);

		$this->db->Execute($query)
			or die ("Error modifying item: " .
	 		$this->db->ErrorMsg() . " " . $query . '<br />');
	}

	function deleteItem($userId, $itemId)
	{
		$item = $this->getItem($userId, $itemId);

		if (isset($item))
		{
			// if the banking information has cancellists attached,
			// delete the cancellists as well (the cancellist will
			// delete its secrets too)
			$this->cancellistServices->deleteItems($userId, $itemId);

			// now delete the specific item
			$query = sprintf($this->queries['deleteItem'], $itemId);

			$result = $this->db->Execute($query) or
				die("BankingServices::DeleteItem: " .
				$this->db->ErrorMsg() . " " . $query. '<br />');
		}
	}

	/**
	 * Delete ALL items for the specified user
	 * @param string userId the name for the user of which all
	 * 	items needs to be deleted
	 */
	function deleteAllItemsForUser($userId)
	{
		// Delete first all secrets and cancellists for the user
		$this->cancellistServices->secretServices->deleteAllItemsForUser($userId);
		$this->cancellistServices->deleteAllItemsForUser($userId);

		// Delete the basic banking information too
		$query = sprintf($this->queries ['deleteAllForUser'], $userId);
		$this->db->Execute ($query);
	}

	function getParent($item)
	{
		return null;
	}

	function getChildren($userId, $itemId)
	{
		if ($itemId == 0)
		{
			$query = sprintf ($this->queries ['getBanks'], $userId);
		}
		else
		{
			$query = sprintf ($this->queries ['getItemChildren'], $itemId, $userId);
		}
		$result = $this->db->Execute ($query) or die
			("Operations. GetChildren " . $this->db->ErrorMsg () .
				" Error getting children: " . $query);
		$items = $this->itemFactory->resultsetToItems($result);
		if ($items == null)
		{
			return null;
		}
		else if ($items[0]->owner == $userId)
		{
			return $items;
		}
		else
		{
			die ("Not owner?");
		}
	}
}
?>
