<?php

require_once 'framework/Installer.php';

/**
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Barry Nauta
 * @package org.brim-project.plugins.bookmarks
 * @subpackage install
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2005 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */
class BookmarkInstaller extends Installer
{
	function BookmarkInstaller ($engine, $db)
	{
		parent::Installer ($engine, $db);
		$this->tableName = 'brim_bookmarks';
	}

	function install ()
	{
		if (!$this->findTable ())
		{
			$dbFile = 'plugins/bookmarks/sql/create.';
			$dbFile .= $this->engine;
			$dbFile .= '.brim_bookmarks.sql';
			$this->createTable ($dbFile);
			$this->createStandardIndex ();
		}
		else
		{
			if (!$this->isTableUpToDate ())
			{
				die ('Table '.$this->tableName.' found but does not appear to be up-to-date and no upgrade procedure is found!');
			}
		}
	}

	function isTableUpToDate ()
	{
		if (!$this->findTable ())
		{
			return false;
		}
		$columnNames = 
			$this->db->MetaColumnNames ('brim_bookmarks', true);
		$metaColumns = $this->db->MetaColumns ('brim_bookmarks', true);
		$locatorColumn = $metaColumns['LOCATOR'];
		$type = $locatorColumn->type;

		return (
			$type != 'varchar' &&
			count ($columnNames) == 14 &&
			$columnNames[0] == 'item_id' &&
			$columnNames[1] == 'owner' &&
			$columnNames[2] == 'parent_id' &&
			$columnNames[3] == 'is_parent' &&
			$columnNames[4] == 'name' &&
			$columnNames[5] == 'description' &&
			$columnNames[6] == 'visibility' &&
			$columnNames[7] == 'category' &&
			$columnNames[8] == 'is_deleted' &&
			$columnNames[9] == 'when_created' &&
			$columnNames[10] == 'when_modified' &&
			$columnNames[11] == 'when_visited' &&
			$columnNames[12] == 'locator' &&
			$columnNames[13] == 'visit_count'
		);
	}
}
?>
