<?php

require_once ('framework/util/StringUtils.php');
require_once ('framework/view/TreeDelegate.php');

/**
 * Used in combination with the Tree renderer class
 *
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Barry Nauta - July 2003
 * @package org.brim-project.plugins.bookmarks
 * @subpackage view
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2005 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */
class BookmarkYahooTreeDelegate extends TreeDelegate
{
	/**
	 * Default constructor
	 */
	function BookmarkYahooTreeDelegate ($theConfiguration)
	{
		parent::TreeDelegate ($theConfiguration);
	}

	/**
	 * Builds up html code to display the root of the tree
	 * @private
	 *
	 * @param object item the item that is the root of the tree
	 */
	function showRoot ($item, $tree)
	{
		if (isset($item) && $item->itemId != 0)
		{
			$resultString  = '<h2>'.$item->name.'</h2>';
			$resultString .= '<a href="'.$this->configuration['callback'];
			$resultString .= '?action=show&parentId='.$item->parentId.'">';
			$resultString .= $this->configuration['icons']['up'];
			$resultString .= '</a><br />';
		}
		else
		{
			$resultString = '';
		}
		return $resultString;
	}

	/**
	 * If the specified item is a folder, draw it
	 * @private
	 * @param object item the item to draw
	 * @param boolean isExpanded whether this folder is expanded (i.e. do the
	 * children of this folder need to be displayed as well?)
	 * @param object tree the tree that uses this class as delegate
	 * @return string the string for this folder
	 */
	function drawFolder ($item, $isExpanded, $tree, $indentLevel)
	{
		//
		// Folder icon
		//
		$resultString  = '<a href="';
		$resultString .= $this->configuration['callback'];
		$resultString .= '?action=modify&itemId=';
		$resultString .= $item->itemId;
        $resultString .= '&parentId=';
		$resultString .= $item->parentId . '">';
		$resultString .= $this->configuration['icons'] ['folder_closed'];
		$resultString .= '</a>';
		//
		// Name
		//
		$resultString .= '<a href="';
		$resultString .= $this->configuration['callback'];
		$resultString .= '?action=show&parentId=';
		$resultString .= $item->itemId . '">';
		$resultString .= $item->name;
		$resultString .= '</a>';
		$resultString .= '&nbsp;';
		$resultString .= '&nbsp;';
		$resultString .= '<br />';
		return $resultString;
	}


	/**
	 * If the specified item is a node, draw it
	 * @private
	 * @param object item the item to draw
	 * @param boolean lastNode whether this node is the last one
	 * @return string the string for this node
	 */
	function drawNode ($item, $lastNode, $tree, $indentLevel)
	{
		//
		// Icon
		//
		$resultString  = '<a href="';
		$resultString .= $this->configuration['callback'];
		$resultString .= '?action=modify&itemId=';
        $resultString .= $item->itemId;
        $resultString .= '&parentId=';
		$resultString .= $item->parentId . '">';
		$resultString .= $this->configuration['icons']['node'];
		$resultString .= '</a>';
		//
		// Link with overlib, alt tag and name
		//
		$resultString .= "<a href='";
		$resultString .= $this->configuration['callback'];
		$resultString .= '?action=showBookmark&itemId=';
		$resultString .= $item->itemId . "' ";
		//
		// Optional Overlib
		//
		if ($this->configuration['overlib'])
		{
			$resultString .= $this->overLib ($item->name, $item->locator.'<br />'.$item->description);
		}
		if (
			isset ($this->configuration['bookmarkNewWindowTarget']) &&
			($this->configuration['bookmarkNewWindowTarget'] == '_blank')
			)
		{
			$resultString .= ' target="'.$this->configuration['bookmarkNewWindowTarget'].'" ';
		}
		//
		// ALT tag
		//
		$resultString .= 'alt="'. $this->stringUtils->urlEncodeQuotes
	   		($this->stringUtils->gpcAddSlashes ($item->name)) . '">';
		$resultString .= $item->name;
		$resultString .= '</a>';
		$resultString .= '<br />';
		return $resultString;
	}
}
?>