<?php

require_once ('framework/model/Services.php');
require_once ('plugins/calendar/model/CalendarFactory.php');

/**
 * Calendar class
 *
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Barry Nauta - December 2003
 * @package org.brim-project.plugins.calendar
 * @subpackage model
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2005 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */
class CalendarServices extends Services
{

	/**
	 * Default constructot
	 */
    function CalendarServices()
    {
	    parent::Services();
		$this->itemFactory = new CalendarFactory ();

		$queries = array ();
		include ('plugins/calendar/sql/calendarQueries.php');
		$this->queries = $queries;
    }


	/**
	 * Get the days for the requested month in an array
	 *
	 * @param integer month the requested month, 1 for Jan.
	 * @param integer year the requested year
	 * @return integer 0 (for Sunday) through 6 (for Saturday)
	 */
	function getFirstDayInMonth ($date)
	{
    	$date = getdate(mktime(12, 0, 0, 
			date ('m', $date), 1, date ('y', $date)));
    	$first = $date["wday"];
    	return $first;
	}

	/**
	 * Returns the number of days in the specified month/year
	 * combination. The year parameter is used for leap-year
	 * calculation
	 *
	 * @param integer month the requested month, 1 for Jan.
	 * @param integer year the requested year
	 * @return integer the number of days in the requested month
	 */
	function getNumberOfDaysInMonthOld ($month, $year)
	{
		if ($this->isLeapYear ($year))
		{
			$numberOfDaysPerMonth =
				array(31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
		}
		else
		{
			$numberOfDaysPerMonth =
				array(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
		}
		return $numberOfDaysPerMonth [$month-1];
	}

	/**
	 * Returns the number of days in the specified month/year
	 * combination. The year parameter is used for leap-year
	 * calculation
	 *
	 * @param integer month the requested month, 1 for Jan.
	 * @param integer year the requested year
	 * @return integer the number of days in the requested month
	 */
	function getNumberOfDaysInMonth ($date)
	{
		return date ('t', $date);
	}

	/**
	 * Is the specified year a leap year?
	 *
	 * @param integer the year for which we would like to know whether
	 * it is a leap-year
	 * @return boolean <code>true</code> if the year is a leap-year,
	 * <code>false</code> otherwise.
	 * @todo replace by date input and format 'L'
	 */
	function isLeapYearOld ($year)
	{
		$result = false;
		if ($year %4 == 0)
		{
			if ($year %100 == 0)
			{
				if ($year %400 == 0)
				{
					$result = true;
				}
			}
			else
			{
				$result = true;
			}
		}
		return $result;
	}


	/**
	 * @param startDayOfWeek 0 for sunday 1 for monday
	 */
	function getMonthsAsArray ($userId, $date, $startDayOfWeek)
	{
		if (gettype ($date) == 'string')
		{
			die ('Unexpected input in getMonthAsArray');
		}
		$year = date ('Y', $date);
		$result = array ();
		for ($i=1; $i<=12; $i++)
		{
			$date= mktime (0, 0, 0, $i, 1, $year); 

			$result [] = $this->getDaysInMonthAsArrayPerWeek 
				($userId, $date, false, $startDayOfWeek);
		}
		return $result;
	}

	/**
	 * @param userId string the identifier (loginname) for the user
	 * @param date integer the internal php date representation
	 * @param requestValues boolean whether we would like to have the 
	 * values for the specific dates retrieved from the database
	 * @param startDayOfWeek 0 for sunday 1 for monday
	 * @return array TBD FIXME
	 */
	function getDaysInMonthAsArrayPerWeek ($userId, $date, $requestValues, $startDayOfWeek)
	{
		if (gettype ($date) == 'string')
		{
			die('Unexpected input getDaysInMonthAsArrayPerWeek:'.$date);
		}

		// 
		// 0 for sunday, 6 for saturday
		//
		$firstDay = $this->getFirstDayInMonth ($date);
		$numberOfDaysInMonth = $this->getNumberOfDaysInMonth ($date);
		$m = date ('m', $date);
		$y = date ('y', $date);	
		// max. 6 (incomplete) weeks in one month
		$weeks = array ();
		for ($dayCount=1, $weekCount=0;
			$dayCount<=$numberOfDaysInMonth;
			$dayCount++)
		{
			//
			// Shortcut. Fool the date'W' function by taking one day 
			// further if the startDay of the week is sunday
			//
			//die (print_r ($startDayOfWeek));
			if ($startDayOfWeek == 0)
			{
				$currentDay = mktime (0, 0, 0, $m, 
					$dayCount + 1, $y);
			}
			else
			{
				$currentDay = mktime (0, 0, 0, $m, 
					$dayCount, $y);
			}
			// 
			// The number of the actual week (between 1 and 53)
			//
			$weekCount2 = date ('W', $currentDay);
			//
			// Reset the day to its actual value so we can 
			// get its events
			//
			$currentDay = mktime (0, 0, 0, $m, $dayCount, $y);
			//
			// Do we want the events for each day as well?
			//
			if ($requestValues)
			{
				$day = array ('day'=>$dayCount, 'events'=>
					$this->getEvents ($userId, $currentDay));
			}
			else
			{
				$day = array ('day'=>$dayCount, 'events'=>null);
			}
			$weeks[$weekCount2][]=$day;
		}
    	return $weeks;
	}

	/**
	 * Adds an event to the calendar plugins
	 *
	 * @param string userId the identifier for the user
	 * @param object item the event itself
	 */
	function addItem ($userId, $item)
	{
			$recurringEndDate = null;
			$eventStartDate = ($item->eventStartDate)?
				date ("Y-m-d H:i", $item->eventStartDate):
					date ('Y-m-d 00:00:00');
			$eventEndDate = ($item->eventEndDate) ?
				date ("Y-m-d H:i", $item->eventEndDate):
					"0000-00-00 00:00:00";
			if ($item->eventRecurringEndDate != -1)
			{
				 $recurringEndDate =
				 	($item->eventRecurringEndDate) ? 
						date ("Y-m-d H:i", $item->eventRecurringEndDate):
							"0000-00-00 00:00:00";

			}
			$now = date  ("Y-m-d H:i:s");
			$query = sprintf ($this->queries['addItem'],
				$userId,
				addslashes ($item->parentId),
				addslashes ($item->isParent),
				addslashes ($item->name),
				addslashes ($item->description),
				addslashes ($item->visibility),
				addslashes ($item->category),
				addslashes ($item->isDeleted),
				addslashes ($now),
				addslashes ($item->location),
				addslashes ($item->organizer),
				addslashes ($item->priority),
				addslashes ($item->frequency),
				addslashes ($item->eventInterval),
				addslashes ($item->byWhat),
				addslashes ($item->byWhatValue),
				$eventStartDate,
				$eventEndDate,
				$recurringEndDate);
			//die ('CalendarServices::addItem '.$query);
			//die (print_r ($query));
			$this->db -> Execute ($query) or die ($db->ErrorMsg ().$query);

			$query = $this->queries['lastItemInsertId'];
			$result=$this->db->Execute($query)
				or die ('AddEvent, could not execute lastItemId: '.$query.'->'.$this->db->ErrorMsg ());
			return $result->fields[0];
	}

	function modifyItem ($userId, $item)
	{
			$eventStartDate = ($item->eventStartDate)?
				date ("Y-m-d H:i", $item->eventStartDate):
					date ('Y-m-d 00:00:00');
			$eventEndDate = ($item->eventEndDate) ?
				date ("Y-m-d H:i", $item->eventEndDate):
					"0000-00-00 00:00:00";
			if ($item->eventRecurringEndDate != -1)
			{
				 $recurringEndDate =
				 	($item->eventRecurringEndDate) ? 
						date ("Y-m-d H:i", $item->eventRecurringEndDate):
							"0000-00-00 00:00:00";

			}
			$now = date  ("Y-m-d H:i:s");
			$now = date  ("Y-m-d H:i:s");
			$query = sprintf ($this->queries['modifyItem'],
				addslashes ($item->parentId),
				addslashes ($item->name),
				addslashes ($item->description),
				addslashes ($item->visibility),
				addslashes ($item->category),
				addslashes ($item->isDeleted),
				addslashes ($now),
				addslashes ($item->location),
				addslashes ($item->organizer),
				$item->priority,
				addslashes ($item->frequency),
				$item->eventInterval,
				addslashes ($item->byWhat),
				addslashes ($item->byWhatValue),
				$eventStartDate,
				$eventEndDate,
				$recurringEndDate,
				$item->itemId);
			$this->db -> Execute ($query) or die ($db->ErrorMsg ());
	}


	/**
	 * @param date string the date in string representation
	 * TODO BARRY FIXME 
	 * This function is called for each date that is being displayed
	 * which may lead to quite some queries to the database when
	 * viewing a month..... We could imagine some serious optimization
	 * here....
	 */
	function getEvents ($userId, $date)
	{
		if (!is_integer($date))
		{
			die ('getEvents did not receive the date as integer!');
		}
		$result = array ();

		$result = array_merge ($result, 
			$this->getYearlyEvents ($userId, $date));
		$result = array_merge ($result, 
			$this->getMonthlyEvents ($userId, $date));	
		$result = array_merge ($result, 
			$this->getWeeklyEvents ($userId, $date));	
		$result = array_merge ($result, 
			$this->getDailyEvents ($userId, $date));	
		$result = array_merge ($result, 
			$this->getDayEvents ($userId, $date));	
			
		//
		// We might doubles (events that show up in the weekly 
		// but also in the monthly)
		//
		$result = $this->uniqueArray ($result);
		return $result;
	}

	/**
	 */
	function getAllEvents ($userId, $date, $dateFrom, $dateTo)
	{
		$result = array ();

		$result = array_merge ($result, 
			$this->getYearlyEvents ($userId, $date));
		$result = array_merge ($result, 
			$this->getMonthlyEvents ($userId, $dateFrom, $dateTo));	
		$result = array_merge ($result, 
			$this->getAllWeeklyEvents ($userId, $dateFrom, $dateTo));	
		$result = array_merge ($result, 
			$this->getDailyEvents ($userId, $dateFrom, $dateTo));	
		$result = array_merge ($result, 
			$this->getDayEvents ($userId, $dateFrom, $dateTo));	
			
		// We might doubles (events that show up in the weekly 
		// but also in the monthly)
		//$result = array_unique ($result);
		$result = $this->uniqueArray ($result);
			//die (print_r ($result));
		return $result;
	}

	function uniqueArray ($input)
	{
		sort ($input);
		reset ($input);
		$result = array ();
		$i = 0;

		$current = current ($input);
		for ($j=0; $j<sizeof ($input); $j++)
		{
			if (next ($input) != $current)
			{
				$result[$i]=$current;
				$current = current ($input);
				$i++;
			}
		}
		return $result;
	}

	/**
	 */
	function getYearlyEvents ($userId, $date)
	{
		$day = date ('d', $date);
		$month = date ('m', $date);
		$startOfDay = mktime (0, 0, 0, 
			date ('m', $date),
			date ('d', $date),
			date ('y', $date));
		$query = sprintf ($this->queries['getYearlyEvents'],
			$userId, date ("Y-m-d H:i:s", $startOfDay), $month, $day, 
				date ('Y-m-d H:i:s', $startOfDay));
			//die ($query);
		$rs = $this->db->Execute ($query) or die ($db->ErrorMsg().' '.$query);
		$result =  $this->itemFactory->resultSetToItems($rs);
		return $result;
	}

	function getMonthlyEvents ($userId, $date)
	{
		// What about monthly events that span multiple days?
		// This function doesn't take that into account!!
		// On the other hand... Should it?
		$day = date ('d', $date);
		$startOfDay = mktime (0, 0, 0, 
			date ('m', $date),
			date ('d', $date),
			date ('y', $date));
		$endOfDay = $startOfDay + (60*60*24) -1;
		$query = sprintf ($this->queries['getMonthlyEvents'],
			$userId, 
			date ("Y-m-d H:i:s", $endOfDay), 
			date ("Y-m-d H:i:s", $startOfDay), 
			$day);
			//die ($query);
		$rs = $this->db->Execute ($query) or die ($db->ErrorMsg().' '.$query);
		return $this->itemFactory->resultSetToItems($rs);
	}

	/**
	 * OBSOLETE
	 */
	function getWeeklyEvents ($userId, $date)
	{
		//die (print_r (date ('Y-m-d', $date)));
		//$this->logger->log ('getWeeklyEvents for date '.date ('Y-m-d', $date));
		$currentWeekDay = date ('w', $date);
		//$currentWeekDay = 6;
		$day = '';
		for ($i=0; $i<$currentWeekDay; $i++)
		{
			$day .= '_';
		}
		$day .= '1';
		for ($j=6; $j>$currentWeekDay; $j--)
		{
			$day .= '_';
		}
		//die ($day);
		$startOfDay = mktime (0, 0, 0, 
			date ('m', $date),
			date ('d', $date),
			date ('y', $date));
		$endOfDay = $startOfDay + (60*60*24) -1;
		// 
		// For the weekly events, take the endOfDay twice (instead
		// of startOfDay). Otherwise, events starting on that specific
		// day, but at a later time than 00:00 will not be counted
		//
		$query = sprintf ($this->queries['getWeeklyEvents'],
			$userId, 
			date ("Y-m-d H:i:s", $endOfDay), 
			date ("Y-m-d H:i:s", $endOfDay), 
			$day);
		//$this->logger->log ($query);
		//die ($query);
		$rs = $this->db->Execute ($query) or die ($db->ErrorMsg().' '.$query);
		return $this->itemFactory->resultSetToItems($rs);
		
	}

/**
 * @obsolete
 */
	function getAllWeeklyEventsObsolete ($userId, $dateFrom, $dateTo)
	{
		//die (print_r (date ('Y-m-d', $date)));
		$date = mktime (0, 0, 0, date('m'), date('d'), date('y'));
		$currentWeekDay = date ('w', $date);
		$currentWeekDay = 6;
		$day = '';
		for ($i=0; $i<$currentWeekDay; $i++)
		{
			$day .= '0';
		}
		$day .= '1';
		for ($j=6; $j>$currentWeekDay; $j--)
		{
			$day .= '0';
		}
		//die ($day);
		$startOfDay = mktime (0, 0, 0, 
			date ('m', $date),
			date ('d', $date),
			date ('y', $date));
		$endOfDay = $startOfDay + (60*60*24) -1;
		$query = sprintf ($this->queries['getWeeklyEvents'],
			$userId, 
			date ("Y-m-d H:i:s", $startOfDay), 
			date ("Y-m-d H:i:s", $endOfDay), 
			$day);
			//die ($query);
		$rs = $this->db->Execute ($query) or die ($db->ErrorMsg().' '.$query);
		return $this->itemFactory->resultSetToItems($rs);
		
	}

	function getDailyEvents ($userId, $date)
	{
		$yesterday = date ("U", ($date - (60*60*24)));
		$startOfDay = mktime (0, 0, 0, 
			date ('m', $date),
			date ('d', $date),
			date ('y', $date));
		//$tomorrow = $date + (60*60*24);
		$endOfDay = $startOfDay + (60*60*24) -1;
		$query = sprintf ($this->queries['getDailyEvents'],
			$userId,
			date ("Y-m-d H:i:s", $endOfDay), 
			date ("Y-m-d H:i:s", $startOfDay));
			//die ($query);
		$rs = $this->db->Execute ($query) or die ($db->ErrorMsg().' '.$query);
			//die (print_r ($rs));
		return $this->itemFactory->resultSetToItems($rs);
	}
	function getDayEvents ($userId, $date)
	{
		$yesterday = date ("U", ($date - (60*60*24)));
		$startOfDay = mktime (0, 0, 0, 
			date ('m', $date),
			date ('d', $date),
			date ('y', $date));
		//$tomorrow = $date + (60*60*24);
		$endOfDay = $startOfDay + (60*60*24) -1;
		$query = sprintf ($this->queries['getEvents'],
			$userId,
			date ("Y-m-d H:i:s", $endOfDay), 
			date ("Y-m-d H:i:s", $startOfDay));
			//die ($query);
		$rs = $this->db->Execute ($query) or die ($db->ErrorMsg().' '.$query);
			//die (print_r ($rs));
		return $this->itemFactory->resultSetToItems($rs);
	}

	function getFirstDayOfWeek ($date, $startDayOfWeek)
	{
		if ($startDayOfWeek == 1)
		{
			$weekday =  (date('w', $date) + 6) % 7;
		}
		else
		{
			$weekday = date ('w', $date);
		}
		return ($date - ($weekday * 24 * 60 * 60));
	}

	function getWeekForDay ($userId, $date, $startDayOfWeek)
	{
			$days = array ();
			$result = array ();

			if ($startDayOfWeek == 1)
			{
				$weekday =  (date('w', $date) + 6) % 7;
			}
			else
			{
				$weekday = date ('w', $date);
			}
			$before = $weekday;
			$after = 6-$weekday;

			for ($i=$before; $i>0; $i--)
			{
					$days [] = mktime(0,0,0, date('m', $date),(date('d', $date)-$i),date('Y', $date));
			}
			$days [] = $date;
			for ($j=1; $j<=$after; $j++)
			{
					$days [] = mktime(0,0,0, date('m', $date),(date('d', $date)+$j),date('Y', $date));
			}
			for ($i=0; $i<count($days); $i++)
			{
				$result[] = $this->getEvents ($userId, $days[$i]);
			}
			return $result;
	}
}
?>
