<?php

require_once ('framework/model/ItemFactory.php');
require_once ('plugins/gmail/model/Message.php');
require_once ('plugins/gmail/model/Conversation.php');
require_once ('framework/util/StringUtils.php');

/**
 * MailFactory
 *
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Barry Nauta - May 2005
 * @package org.brim-project.plugins.gmail
 * @subpackage model
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2005 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */
class MailFactory  extends ItemFactory
{
	var $stringUtils;
	/**
	 * Default constructor
	 */
	function MailFactory ()
	{
		parent::ItemFactory ();
		$this->stringUtils = new StringUtils ();
	}
	
	function getMailFromGMailSnapshot ($gmailSnapshot)
	{
		//die (print_r ($gmailSnapshot));
		$messages = array ();
		foreach ($gmailSnapshot->conv as $gmessage)
		{
			$message = $this->getEmptyMessage ();
			$message->messageId = $gmessage['id'];
			$message->sender = $this->stringUtils->removeBizarreCharacters(
				$gmessage['sender']);
			$message->from = $gmessage['sender_email'];
			$message->subject = $this->stringUtils->removeBizarreCharacters(
				$gmessage['subj']);
			$message->date = $snapshot['date'];
			if (isset ($gmessage['body']) && trim($gmessage['body'])!='')
			{
				$message->body = $this->stringUtils->
					removeBizarreCharacters ($gmessage['body']);
			}
			else
			{
				$message->body = $this->stringUtils->
					removeBizarreCharacters ($gmessage['snippet']);
			}
			$messages [] = $message;		
			// TODO More
		}
		return $messages;
	}

	function getConversationFromGMailSnapshot ($gmailSnapshot)
	{
		//die (print_r ($gmailSnapshot));
		$conversation = new Conversation 
			($gmailSnapshot->conv_title, $gmailSnapshot->conv_id);
		$conversation->setLabels ($gmailSnapshot->labels);
		$conversation->setStarred ($gmailSnapshot->conv_starred);

		//die (print_r ($gmailSnapshot));
		//if (isset ($gmailSnapshot->conv) && is_array ($gmailSnapshot->conv))
		//{
			foreach ($gmailSnapshot->conv as $gmessage)
			{
				//die (print_r ($gmessage));
				$message = $this->getEmptyMessage ();
				$message->messageId = $gmessage['id'];
				$message->sender = $this->stringUtils->removeBizarreCharacters(
					$gmessage['sender']);
				$message->from = $gmessage['sender_email'];
				$message->subject = $this->stringUtils->removeBizarreCharacters(
					$gmessage['subj']);
				$message->date = $snapshot['date'];
				foreach ($gmessage['attachment'] as $attachment)
				{
					$message->attachments[]=$attachment;
				}
				if (isset ($gmessage['body']) && trim($gmessage['body'])!='')
				{
					$message->body = $this->stringUtils->
						removeBizarreCharacters ($gmessage['body']);
				}
				else
				{
					$message->body = $this->stringUtils->
						removeBizarreCharacters ($gmessage['snippet']);
				}
				$conversation->addMessage ($message);
			}
		//}
		//else
		//{
			//die (print_r ($gmailSnapshot));
			//$conversation = null;
		//}
		return $conversation;
	}

	function getAllMailFromGMailSnapshot ($gmailSnapshot)
	{
		//die (print_r ($gmailSnapshot));
		$messages = array();
		$message = null;
		for ($i=0; $i<count($gmailSnapshot->box); $i++)
		{
			$snapshot = $gmailSnapshot->box[$i];
			//die (print_r ($snapshot));
			$message = $this->getEmptyMessage ();
			$message->from = $this->stringUtils->removeBizarreCharacters(
				$snapshot['sender']);
			$message->isRead = $snapshot['is_read'];
			$message->date = $snapshot['date'];
			$message->subject=$this->stringUtils->removeBizarreCharacters(
				$snapshot['subj']);
			$message->body = $snapshot['snippet'];
			$message->flags[] = $snapshot['flag'];
			$message->attachments = $snapshot['attachment'];
			if (isset ($snapshot['is_starred']) 
				&& ($snapshot['is_starred'] == 1))
			{
				$message->flags[] = 'starred';
			}
			$message->messageId = $snapshot['msgid'];
			$messages [] = $message;
		}
		return $messages;
	}

	/**
	 * Returns the type of this specific item
	 * @return string the type of this specific 
	 * item: <code>Note</code>
	 */
	function getType ()
	{
		return "Message";
	}
	
	function getEmptyMessage ()
	{
		return new Message (null, null,
			null, null, null, null,
			null, null, null);
	}
}
?>
