<?php

require_once ("framework/util/StringUtils.php");
require_once ("framework/view/TreeDelegate.php");

/**
 * Used in combination with the Tree class
 *
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Barry Nauta - January 2004
 * @package org.brim-project.plugins.tasks
 * @subpackage view
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2005 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */
class TaskLineBasedTreeDelegate extends TreeDelegate
{
	/**
	 * Have we already drawn a header?
	 */
	var $headersDrawn;

	function TaskLineBasedTreeDelegate ($theConfiguration)
	{
		parent::TreeDelegate ($theConfiguration);
		$this->headersDrawn = false;
	}
	
	/**
	 * Builds up html code to display the root of the tree
	 * @private
	 *
	 * @param object item the item that is the root of the tree
	 */
	function showRoot ($item, $tree)
	{
		$resultString = '';
		if (isset($item) && $item->itemId != 0)
		{
			$resultString .= '<h2>'.$item->name.'</h2>';
			$resultString .= '<a href="'.$this->configuration['callback'];
			$resultString .= '&action=show&parentId='.$item->parentId.'">';

			$resultString .= $this->configuration['icons']['up'];
			$resultString .= '</a><br />';
		}
		else
		{
			$resultString = '';
		}
		return $resultString;
	}
	
	/**
	 * If the specified item is a folder, draw it
	 * @private
	 * @param object item the item to draw
	 * @param boolean isExpanded whether this folder is expanded (i.e. do the
	 * children of this folder need to be displayed as well?)
	 * @param object tree the tree that uses this class as delegate
	 * @return string the string for this folder
	 */
	function drawFolder ($item, $isExpanded, $tree, $indentLevel)
	{
		$resultString = '';
		$resultString .= '<a href="';
		$resultString .= $this->configuration['callback'];
        $resultString .= '&action=modify&itemId='.$item->itemId;
        $resultString .= '&parentId='.$item->parentId.'">';
		$resultString .= $this->configuration['icons']['folder_closed'];
		$resultString .= '</a>';

		$resultString .= '<a href="';
		$resultString .= $this->configuration['callback'];
		$resultString .= '&action=show&parentId=';
		$resultString .= $item->itemId . '">';
		$resultString .= $item->name;
		$resultString .= '</a>';
		
		$resultString .= '<br />';
		return $resultString;
	}
	
	function drawLinesHeader ($parentId)
	{
		$resultString = '<td colspan="8"></tr><tr>';
		$resultString .= '<td>&nbsp;</td>';
		$resultString .= '<td>&nbsp;</td>';

		$resultString .= '<td><b>'.$this->configuration['dictionary']['name'];
		$resultString .= $this->sortArrows ('name', $parentId);
		$resultString .= '</b></td>';
		
		$resultString .= '<td><b>'.$this->configuration['dictionary']['priority'];
		$resultString .= $this->sortArrows ('priority', $parentId);
		$resultString .= '</b></td>';

		$resultString .= '<td><b>'.$this->configuration['dictionary']['status'];
		$resultString .= $this->sortArrows ('status', $parentId);
		$resultString .= '</b></td>';

		$resultString .= '<td><b>'.$this->configuration['dictionary']['complete'];
		$resultString .= $this->sortArrows ('percentComplete', $parentId);
		$resultString .= '</b></td>';

		$resultString .= '<td><b>'.$this->configuration['dictionary']['start_date'];
		$resultString .= $this->sortArrows ('start_date', $parentId);
		$resultString .= '</b></td>';

		$resultString .= '<td><b>'.$this->configuration['dictionary']['due_date'];
		$resultString .= $this->sortArrows ('end_date', $parentId);
		$resultString .= '</b></td>';
		
		$resultString .= '</tr><tr class="odd">';
		return $resultString;
	}
	/**
	 * If the specified item is a node, draw it
	 * @private
	 * @param object item the item to draw
	 * @param boolean lastNode whether this node is the last one
	 * @return string the string for this node
	 */
	function drawNode ($item, $lastNode, $tree, $indentLevel)
	{
		if ($this->headersDrawn)
		{
			$resultString = '';
		}
		else
		{
			$resultString = $this->drawLinesHeader ($item->parentId);
			$this->headersDrawn = true;
		}
		// header with two icons and overlib popup
		$resultString .= '<td width="20" align="right">';
		$resultString .= '<a href="'.$this->configuration['callback'];
        $resultString .= '&action=modify&itemId='.$item->itemId;
        $resultString .= '&parentId='.$item->parentId.'">';
		$resultString .= $this->configuration['icons']['edit'].'</a>';
		$resultString .= '</td>';
		
		$resultString .= '<td width="20" align="right">';
		$resultString .= '<a href="'.$this->configuration['callback'];
		$resultString .= '&itemId='.$item->itemId;
		$resultString .= '&parentId='.$item->parentId;
		$resultString .= '&action=deleteItemPost" ';
		// delete confirmation
		$resultString .= 'onclick="javascript:return confirm (\'';
		$resultString .= $this->configuration['dictionary']['confirm_delete'];
		$resultString .= '\');" ';

		$resultString .= '>'.$this->configuration['icons']['delete'].'</a>';
		$resultString .= '</td>';

		$resultString .= '<td>';
		$resultString .= '<a href="'.$this->configuration['callback'];
        $resultString .= '&action=showItem&itemId='.$item->itemId;
        $resultString .= '&parentId='.$item->parentId.'" ';
		if ($this->configuration['overlib'])
		{
			$resultString .= $this->overLib ($item->name, $item->description);
		}
		$resultString .= '>';
		$resultString .= '<b>'.$this->stringUtils->truncate ($item->name, 30). '</b></a>';
		$resultString .= '</td>';
		
		$resultString .= '<td>'.$this->configuration['dictionary']['priority'.$item->priority].'</td>';
		$resultString .= '<td>';
		if (isset ($item->status))
		{
			$resultString .= $item->status;
		}
		else
		{
			$resultString .= '&nbsp;';
		}
		$resultString .= '</td>';
		$resultString .= '<td>'.$item->percentComplete.'</td>';
		if (isset ($item->startDate))
		{
			$resultString .= '<td>'.
				date ('Y-m-d', strtotime ($item->startDate)).'</td>';
		}
		else
		{
			$resultString .='<td>&nbsp;</td>';
		}
		$resultString .= '<td>';
		if (isset ($item->endDate) && 
			($item->endDate < date("Y-m-d H:i:s")))
		{
			$resultString .= '<font color="ff0000">';
			$resultString .= date ('Y-m-d', strtotime ($item->endDate));
			$resultString .= '</font>';
		}
		else
		{
			if (isset ($item->endDate))
			{
				$resultString .= date ('Y-m-d', 
					strtotime ($item->endDate));
			}
			else
			{
				$resultString .= '<td>&nbsp;</td>';
			}
		}
		$resultString .= '</td>';
		return $resultString;
	}
	
}
?>
