<?php

require_once ('framework/util/databaseConnection.php');
include_once ('framework/sql/authQueries.php');
require_once ('framework/model/User.php');
require_once ('framework/model/UserFactory.php');
require_once ('framework/model/Preference.php');
require_once ('framework/model/UserServices.php');
require_once ('framework/model/PreferenceServices.php');
require_once ('framework/model/AdminServices.php');

/**
 * This file is part of the Brim project.
 * The brim-project is located at the following
 * location: {@link http://www.brim-project.org/ http://www.brim-project.org/}
 *
 * <pre> Enjoy :-) </pre>
 *
 * @author Barry Nauta 
 * @package org.brim-project.framework
 *
 * @copyright [brim-project.org] - Copyright (c) 2003 - 2005 Barry Nauta
 *
 * @license http://opensource.org/licenses/gpl-license.php
 * The GNU Public License
 */
$defaultLanguage = 'en';
$defaultTemplate = 'barrel';
$userFactory = new UserFactory ();
$adminServices = new AdminServices ();
$allowSignon = $adminServices->getAdminConfig ('allow_account_creation');
if (!allowSignon)
{
	//
	// We can only request this is allowed by admin
	// 
	if(!$_POST['loginName'] | !$_POST['password'])
	{
		$message='illegalAccess';
		Header ("Location: login.php?message=".$message);
		exit;
	}
}
if (isset($_POST['submit']))
{
	//
	// Check for username and password on POST. If this is not set, 
	// there was an  attempt -> illegal access?
	//
	if(!$_POST['loginName'] | !$_POST['password'])
	{
		$message='illegalAccess';
		Header ("Location: login.php?message=".$message);
		exit;
	}
	//
	// Check for illegal characters (', &, %, $, #) in the loginName
	//
	if (
			strstr ($_POST['loginName'], "'") ||
			strstr ($_POST['loginName'], "&") ||
			strstr ($_POST['loginName'], "%") ||
			strstr ($_POST['loginName'], "$") ||
			strstr ($_POST['loginName'], "#")
		)
	{
		$message='illegalLoginName';
		Header ("Location: login.php?message=".$message);
		exit;
	}
	//
	// Check whether the two provided passwords match
	//
	if ($_POST['password'] != $_POST['password2'])
	{
		$message = 'passwordMismatch';
		Header ("Location: login.php?message=".$message);
		exit;
	} 
	//
	// Query the database to see whether the requested id for the user
	// does not already exist in the database
	//
	$query = sprintf 
		($queries ['getUsernamePassword'], $_POST['loginName']);
	$result = $db->Execute($query) or die ($db->ErrorMsg ());
	$dbUsername = $result->fields[0];
	//
	// Check whether we have received a username from the database. 
	// If this is the case, the username already exists and cannot 
	// be added
	//
	if (isset ($dbUsername) || $dbUsername != "")
	{
		$message='userAlreadyExists';
		Header ("Location: login.php?message=".$message);
		exit;
	}
	$adminEmail = $adminServices->getAdminConfig ('admin_email');
	//
	// Now physically add the user and its preferences.
	// We can now also savely set the username on the session so the 
	// user is directly logged in.
	/// 
	$userSettings = $userFactory->requestToUser ();
	$preferencesServices = new PreferenceServices ();
	$userServices = new UserServices ();
	session_start();
	$_SESSION['brimUsername'] = $userSettings->loginName;
	$languagePreference = new Preference 
			(null, $_SESSION['brimUsername'], 0, 0, 
				'brimLanguage', null, 'private', null, 0, null, 
					null, $defaultLanguage);
	$templatePreference = new Preference 
			(null, $_SESSION['brimUsername'], 0, 0, 
				'brimTemplate', null, 'private', null, 0, null, 
					null, $defaultTemplate);
	$preferencesServices->addItem 
		($_SESSION['brimUsername'], $languagePreference);
	$preferencesServices->addItem 
		($_SESSION['brimUsername'], $templatePreference);
	$userServices->addUser ($_SESSION['brimUsername'], $userSettings);
	//
	// Ok, username does not exist and credentials (passwords) appear 
	// to be valid. Send a mail to the site admin
	//
	$message = "A new user (".$_POST['loginName'].") just signed in";
	$message .= "
		Address: ".$_SERVER['REMOTE_ADDR'];
	$subject = "New user (".$_POST['loginName'].") addition";
	$headers = "MIME-Version: 1.0\r\n";
	$headers .= "Content-type: text/html; charset=iso-8859-1\r\n";
	$headers .= "From: Brim administrator <".$adminEmail.">\r\n";
	$headers .= "To: Brim administrator <".$adminEmail.">\r\n";
	$headers .= "X-Mailer: Brim";
	mail($adminEmail, $subject, $message, $headers);
	//
	// Forward to the login page with an appropriate message
	//
	Header ("Location: index.php");
	exit;
}
?>
